@@ -639,7 +639,7 @@ static void pick_external(P_CmdSeq *page)
 
 	scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	clist = clist = gtk_clist_new(2);
+	clist = gtk_clist_new(2);
 	gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
 	gtk_clist_set_column_width(GTK_CLIST(clist), 0, 160);
 	gtk_clist_set_column_width(GTK_CLIST(clist), 1, 288);
