@@ -306,6 +306,7 @@ static gboolean is_parent_of(const char *from, const char *to)
 */
 gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 {
+	gboolean	aborted = FALSE;
 	gchar		old_dir[PATH_MAX];
 	DIR		*dir;
 	struct dirent	*de;
@@ -352,7 +353,7 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 		{
 			if((dir = opendir(".")) != NULL)
 			{
-				while(!errno && (de = readdir(dir)) != NULL)
+				while(!aborted && !errno && (de = readdir(dir)) != NULL)
 				{
 					struct stat	stat;
 
@@ -366,12 +367,12 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 						if(S_ISDIR(stat.st_mode))
 						{
 							if(!copy_dir(min, de->d_name, dest2))
-								break;
+								aborted = TRUE;
 						}
 						else
 						{
 							if(!copy_file(min, de->d_name, dest2, &stat))
-								break;
+								aborted = TRUE;
 						}
 					}
 				}
@@ -384,7 +385,7 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 	pgs_progress_item_end(min);
 	if(errno)
 		err_set(min, errno, CMD_ID, from);
-	return (errno == 0) ? TRUE : FALSE;
+	return (errno == 0 && !aborted) ? TRUE : FALSE;
 }
 
 /* ----------------------------------------------------------------------------------------- */
