@@ -255,6 +255,7 @@ typedef struct {
 } DPPaning;
 
 typedef struct {
+	gboolean	select;			/* Rememeber selections? */
 	gboolean	save;			/* Save history lists on exit? */
 } DPHistory;
 
@@ -367,7 +368,7 @@ typedef struct {			/* Holds all configuration info. */
 	DPFormat	dp_format[2];
 	DPColors	dp_colors;		/* Global color information used to render dirpanes. */
 	DPPaning	dp_paning;		/* Controls how the paned view acts. */
-	DPHistory	dp_history;		/* History list option. */
+	DPHistory	dp_history;		/* History list options. */
 	OptOverwrite	opt_overwrite;
 	MenuInfo	*menus;
 	ButtonInfo	buttons;
@@ -384,7 +385,7 @@ typedef struct {			/* Holds all configuration info. */
 
 typedef struct GuiInfo	GuiInfo;
 
-struct MainInfo {				/* gentoo's single most central data structure. */
+struct MainInfo {				/* gentoo's single most important data structure. It's everywhere. */
 	OptDesc		*opt;
 	gsize		opt_num;
 	CfgInfo		cfg;
