@@ -18,6 +18,7 @@
 
 #include "dialog.h"
 #include "dirpane.h"
+#include "miscutil.h"
 
 #include "gfam.h"
 
@@ -88,12 +89,18 @@ static gboolean evt_fam_timeout(gpointer data)
 	return FALSE;
 }
 
-/* 2003-09-30 -	Rate limit pane rescans. Returns TRUE if pane <index> can do a rescan, FALSE
-**		if it cannot.
+/* Refresh rate period is always in this range, in seconds. */
+#define	LIMIT_MIN	0.3f
+#define	LIMIT_MAX	2.0f
+
+/* 2003-09-30 -	Rate limit pane rescans. Tracks elaped time since last rescan, and limit the frequency of
+**		actual rescans performed. Returns TRUE if it did do the rescan, FALSE if not. The rate of
+**		rescanning is dynamically altered based on how long it takes to do; aiming to keep the a
+**		1:4 ratio between time to rescan and time between rescans.
 */
-static gboolean rescan_rate_limit(guint index)
+static gboolean rescan_rate_limited(guint index)
 {
-	const gfloat LIMIT = 0.300f;			/* Lower limit on period between refreshes. */
+	static gfloat	limit = LIMIT_MIN;			/* Dynamically adjusted based on refresh time. */
 	static GTimeVal stamp[2] = { { 0 }, { 0 } };
 	static gfloat elapsed[2] = { 0.0f, 0.0f};		/* Time since last rescan allowed. */
 	GTimeVal now;
@@ -102,23 +109,33 @@ static gboolean rescan_rate_limit(guint index)
 	g_get_current_time(&now);
 	if(stamp[index].tv_sec && stamp[index].tv_usec)
 	{
-		gfloat dt = 1E-6f * (now.tv_usec - stamp[index].tv_usec) + (now.tv_sec - stamp[index].tv_sec);
-		if(dt < LIMIT)
+		gfloat dt = msu_diff_timeval(stamp + index, &now);
+		if(dt < limit)
 		{
 			elapsed[index] += dt;
-			if(elapsed[index] < LIMIT)	/* When blocking has reached limit, let a request through. */
+			if(elapsed[index] < limit)	/* When blocking has reached limit, let a request through. */
 			{
 				if(the_faminfo.timeout[index])
 					gtk_timeout_remove(the_faminfo.timeout[index]);
-				the_faminfo.timeout[index] = gtk_timeout_add(1000.0f * LIMIT, evt_fam_timeout, GUINT_TO_POINTER(index));
+				the_faminfo.timeout[index] = gtk_timeout_add(1000.0f * limit, evt_fam_timeout, GUINT_TO_POINTER(index));
 				ret = FALSE;
 			}
 		}
 	}
 	stamp[index] = now;
 
-	if(ret)	/* If we're allowing refresh, cancel any outstanding timer and reset bookkeeping. */
+	if(ret)	/* Refresh needed; do it, measure how long it takes, and update rate. */
 	{
+		gfloat	dt;
+
+		dp_rescan(&the_faminfo.min->gui->pane[index]);
+		g_get_current_time(&now);
+		dt = msu_diff_timeval(&stamp[index], &now);
+		limit = 3.0f * dt;		/* 3 + 1 is four. */
+		if(limit < LIMIT_MIN)
+			limit = LIMIT_MIN;
+		else if(limit > LIMIT_MAX)
+			limit = LIMIT_MAX;
 		elapsed[index] = 0.0f;
 		if(the_faminfo.timeout[index])
 		{
@@ -136,10 +153,13 @@ static void rescan_issue(guint rescan)
 
 	for(i = 0; i < sizeof the_faminfo.req / sizeof *the_faminfo.req; i++)
 	{
-		if(rescan & (1 << i) && rescan_rate_limit(i))
+		if(rescan & (1 << i))
 		{
-			dp_rescan(&the_faminfo.min->gui->pane[i]);
-			f |= the_faminfo.min->gui->cur_pane == &the_faminfo.min->gui->pane[i];
+			if(rescan_rate_limited(i))
+			{
+			/*	dp_rescan(&the_faminfo.min->gui->pane[i]);*/
+				f |= the_faminfo.min->gui->cur_pane == &the_faminfo.min->gui->pane[i];
+			}
 		}
 	}
 	if(f)
