@@ -36,6 +36,8 @@
 #define	HEX_ROW_SIZE	(16)		/* Must be multiple of four! */
 #define	HEX_UNPRINTABLE	('.')
 
+#define	LABEL_LIMIT	(64)		/* Keep reasonably large. */
+
 /* ----------------------------------------------------------------------------------------- */
 
 typedef enum { MVE_UP = 0, MVE_DOWN, MVE_TOP, MVE_BOTTOM } MoveID;
@@ -77,8 +79,17 @@ static DlgSrch	srch_dialog = { NULL };		/* Same here. */
 
 #define	SEARCH_RE_SIZE	(256)			/* Far more than *I* would need... ;^) */
 
-static gchar	last_re[SEARCH_RE_SIZE] = "";	/* Buffer the RE between searches. */
-static gint	last_fnocase = 0, last_fnolf = 0;
+/* Store search parameters between invocations. */
+static struct {
+	gchar		re[SEARCH_RE_SIZE];
+	gboolean	no_case;
+	gboolean	no_lf;
+} last_search = { "", FALSE, FALSE };
+
+/* ----------------------------------------------------------------------------------------- */
+
+static gboolean	do_search(TxvInfo *txi, const gchar *re_src, gboolean nocase, gboolean nolf);
+static void	do_search_repeat(TxvInfo *txi);
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -111,28 +122,6 @@ static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 	return TRUE;
 }
 
-/* 1999-02-23 -	An attempt to support ESCape for closing down the text viewer. Shouldn't
-**		be too harmful. I hope.
-*/
-static gint evt_keypress(GtkWidget *wid, GdkEventKey *event, gpointer user)
-{
-	TxvInfo	*txi = gtk_object_get_user_data(GTK_OBJECT(wid));
-
-	if(event->keyval == GDK_Escape)
-		really_destroy(wid);
-	else if(event->keyval == GDK_Home)
-	{
-		GtkAdjustment	*adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin));
-		gtk_adjustment_set_value(adj, 0.0f);
-	}
-	else if(event->keyval == GDK_End)
-	{
-		GtkAdjustment	*adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin));
-		gtk_adjustment_set_value(adj, adj->upper);
-	}
-	return FALSE;
-}
-
 /* 1999-03-02 -	Users of this module no longer have access to the GtkText widget, so
 **		this function is needed for them to be able to actually display text.
 */
@@ -241,8 +230,8 @@ gint txv_load_text_hex(GtkWidget *win, const gchar *name)
 	gchar		line[8 + 1 + (HEX_ROW_SIZE / 4) * 9 + 1 + HEX_ROW_SIZE + 1 + 1],
 			group[9 * (HEX_ROW_SIZE / 4) + 1], text[HEX_ROW_SIZE + 1];
 	guchar		buf[HEX_ROW_SIZE];
-	gint		fd, len;
-	gsize		chunk, offset = 0;
+	gint		fd, len, chunk;
+	gsize		offset = 0;
 	TxvInfo		*txi;
 
 	if((win == NULL) || ((txi = gtk_object_get_user_data(GTK_OBJECT(win))) == NULL) || (name == NULL))
@@ -295,6 +284,35 @@ static void reset_search(TxvInfo *txi)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 1999-02-23 -	An attempt to support ESCape for closing down the text viewer. Shouldn't
+**		be too harmful. I hope.
+*/
+static gint evt_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
+{
+	TxvInfo	*txi = gtk_object_get_user_data(GTK_OBJECT(wid));
+
+	if(evt->keyval == GDK_Escape)
+		really_destroy(wid);
+	else if(evt->keyval == GDK_End)
+	{
+		GtkAdjustment	*adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin));
+		gtk_adjustment_set_value(adj, adj->upper);
+	}
+	else if(evt->keyval == GDK_F3)
+		do_search_repeat(txi);
+	else
+		return FALSE;
+	return TRUE;
+}
+
+/* 2004-04-09 -	Close on right-button press. */
+static gint evt_button_press(GtkWidget *wid, GdkEventButton *evt, gpointer user)
+{
+	if(evt->type == GDK_BUTTON_PRESS && evt->button == 3)
+		really_destroy(wid);
+	return FALSE;
+}
+
 /* 1999-01-07 -	This handler gets called when the vertical range (scrollbar) changes. This
 **		should probably also include the number of lines available, but that is a
 **		rather fuzzy quantity, so I'll just leave that for now.
@@ -435,13 +453,13 @@ static void show_text(TxvInfo *txi, gint pos)
 {
 	GtkAdjustment	*adj;
 	GtkStyle	*stl;
-	gchar		*text;
+	gchar		*base, *text;
 	gfloat		nv;
 	gint		line, fh = 10;
 
-	if((text = gtk_editable_get_chars(GTK_EDITABLE(txi->text), 0, -1)) != NULL)
+	if((base = gtk_editable_get_chars(GTK_EDITABLE(txi->text), 0, -1)) != NULL)
 	{
-		for(line = 0; *text && pos; pos--, text++)
+		for(line = 0, text = base; *text && pos; pos--, text++)
 		{
 			if(*text == '\n')		/* Probably breaks on non-Unix systems. So? */
 				line++;
@@ -456,19 +474,20 @@ static void show_text(TxvInfo *txi, gint pos)
  					gtk_adjustment_set_value(adj, nv);
 			}
 		}
+		g_free(base);
 	}
 }
 
 /* 1999-01-07 -	Do the actual search, using <re_src> as the regular expression source. This
-**		should really begin with '(', and with ')', and don't contain any non-escaped
+**		should really begin with '(', end with ')', and don't contain any non-escaped
 **		parentheses within!
 */
-static void do_search(TxvInfo *txi, const gchar *re_src, gboolean nocase, gboolean nolf)
+static gboolean do_search(TxvInfo *txi, const gchar *re_src, gboolean nocase, gboolean nolf)
 {
 	regex_t		re;
 	regmatch_t	rm[1];
 	gchar		*text, ebuf[1024];
-	gint		cerr, cflags = REG_EXTENDED;
+	gint		found = FALSE, cerr, cflags = REG_EXTENDED;
 
 	if(nocase)
 		cflags |= REG_ICASE;
@@ -485,7 +504,9 @@ static void do_search(TxvInfo *txi, const gchar *re_src, gboolean nocase, gboole
 				gtk_editable_select_region(GTK_EDITABLE(txi->text), txi->search_pos + rm[0].rm_so,
 								txi->search_pos + rm[0].rm_eo);
 				txi->search_pos += rm[0].rm_eo;
+				found = TRUE;
 			}
+			g_free(text);
 		}
 		regfree(&re);
 	}
@@ -495,6 +516,7 @@ static void do_search(TxvInfo *txi, const gchar *re_src, gboolean nocase, gboole
 		regerror(cerr, &re, ebuf + strlen(ebuf), sizeof ebuf - strlen(ebuf) - 1);
 		dlg_dialog_async_new_error(ebuf);
 	}
+	return found;
 }
 
 /* 1999-01-07 -	User hit the "Search" button, so let's pop up something to request parameters. */
@@ -506,15 +528,15 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 	srch_dialog.label = gtk_label_new(_("Enter Text to Search For (RE)"));
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.label, FALSE, FALSE, 0);
 	srch_dialog.entry = gtk_entry_new_with_max_length(SEARCH_RE_SIZE - 1);
-	gtk_entry_set_text(GTK_ENTRY(srch_dialog.entry), last_re);
+	gtk_entry_set_text(GTK_ENTRY(srch_dialog.entry), last_search.re);
 	gtk_editable_select_region(GTK_EDITABLE(srch_dialog.entry), 0, -1);
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.entry, FALSE, FALSE, 0);
 	srch_dialog.hbox  = gtk_hbox_new(FALSE, 0);
 	srch_dialog.fnocase = gtk_check_button_new_with_label(_("Ignore Case?"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(srch_dialog.fnocase), last_fnocase);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(srch_dialog.fnocase), last_search.no_case);
 	gtk_box_pack_start(GTK_BOX(srch_dialog.hbox), srch_dialog.fnocase, TRUE, TRUE, 0);
 	srch_dialog.fnolf = gtk_check_button_new_with_label(_("Don't Span Newlines?"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(srch_dialog.fnolf), last_fnolf);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(srch_dialog.fnolf), last_search.no_lf);
 	gtk_box_pack_start(GTK_BOX(srch_dialog.hbox), srch_dialog.fnolf, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.hbox, FALSE, FALSE, 0);
 
@@ -527,9 +549,9 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 		GString	*tmp;
 
 		txt = gtk_entry_get_text(GTK_ENTRY(srch_dialog.entry));
-		stu_strncpy(last_re, txt, sizeof last_re);
-		last_fnocase = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(srch_dialog.fnocase));
-		last_fnolf   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(srch_dialog.fnolf));
+		stu_strncpy(last_search.re, txt, sizeof last_search.re);
+		last_search.no_case = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(srch_dialog.fnocase));
+		last_search.no_lf   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(srch_dialog.fnolf));
 		if(txt != NULL && (tmp = g_string_new("(")) != NULL)
 		{
 			for(; *txt; txt++)		/* Quote any parentheses in user's string. */
@@ -542,7 +564,7 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 					g_string_append_c(tmp, *txt);
 			}
 			g_string_append_c(tmp, ')');
-			do_search(txi, tmp->str, last_fnocase, last_fnolf);
+			do_search(txi, tmp->str, last_search.no_case, last_search.no_lf);
 			g_string_free(tmp, TRUE);
 		}
 	}
@@ -553,6 +575,20 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 	}
 }
 
+static void do_search_repeat(TxvInfo *txi)
+{
+	if(last_search.re[0] != '\0')
+	{
+		if(!do_search(txi, last_search.re, last_search.no_case, last_search.no_lf) && txi->search_pos > 0)
+		{
+			reset_search(txi);
+			do_search(txi, last_search.re, last_search.no_case, last_search.no_lf);
+		}
+	}
+	else
+		evt_search_clicked(NULL, txi);
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-10 -	A complete rewrite of the textview widget creation stuff. Now supports just creating
@@ -568,6 +604,8 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 {
 	GtkWidget	*vbox, *btn, *algn, *bhbox;
 	TxvInfo		*txi;
+	GtkAccelGroup	*accel;
+	guint		key;
 
 	txi = g_malloc(sizeof *txi);
 
@@ -578,6 +616,7 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	txi->win = win_window_open(min->cfg.wininfo, WIN_TEXTVIEW);
 	gtk_object_set_user_data(GTK_OBJECT(txi->win), txi);
 	gtk_signal_connect(GTK_OBJECT(txi->win), "delete_event", GTK_SIGNAL_FUNC(evt_delete), txi);
+	gtk_signal_connect(GTK_OBJECT(txi->win), "button_press_event", GTK_SIGNAL_FUNC(evt_button_press), txi);
 	txv_connect_keypress(txi->win, NULL, NULL);
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -609,6 +648,8 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	txi->bplabel = gtk_label_new("");
 	gtk_container_add(GTK_CONTAINER(algn), txi->bplabel);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), algn, TRUE, TRUE, 0);
+	
+	accel = gtk_accel_group_new();
 
 	btn = gui_arrow_button_new(GTK_ARROW_UP);
 	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_UP));
@@ -621,27 +662,37 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	btn = gtk_button_new_with_label(_("Top"));
 	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_TOP));
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
+	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_Home, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 	btn = gtk_button_new_with_label(_("Bottom"));
 	gtk_object_set_user_data(GTK_OBJECT(btn), GINT_TO_POINTER(MVE_BOTTOM));
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_move_clicked), txi);
+	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_End, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 
 	bhbox = gtk_hbox_new(FALSE, 0);
-	btn = gtk_button_new_with_label(_("Goto..."));
+	btn = gtk_button_new_with_label("");
+	key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(btn)->child), _("_Goto..."));
+	gtk_widget_add_accelerator(btn, "clicked", accel, key, 0, 0);
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_goto_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
-	btn = gtk_button_new_with_label(_("Search..."));
+	btn = gtk_button_new_with_label("");
+	key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(btn)->child), _("_Search..."));
+	gtk_widget_add_accelerator(btn, "clicked", accel, key, 0, 0);
+	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_F, GDK_CONTROL_MASK, 0);
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_search_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), bhbox, FALSE, FALSE, 5);
 
-	btn = gtk_button_new_with_label(_("Close"));
+	btn = gtk_button_new_with_label("");
+	key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(btn)->child), _("_Quit"));
+	gtk_widget_add_accelerator(btn, "clicked", accel, key, 0, 0);
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_close_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, TRUE, 0);
 	gtk_widget_set_sensitive(txi->bhbox, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), txi->bhbox, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(txi->win), vbox);
+	gtk_window_add_accel_group(GTK_WINDOW(txi->win), accel);
 	gtk_widget_show_all(vbox);
 
 	gtk_widget_grab_focus(txi->text);
@@ -689,6 +740,26 @@ void txv_set_label(GtkWidget *wid, const gchar *text)
 
 	if(wid != NULL && (txi = gtk_object_get_user_data(GTK_OBJECT(wid))) != NULL && text != NULL)
 	{
+		guint	tl = strlen(text);
+
+		/* If wanted label is very long, cut it down to avoid window resizing. */
+		if(tl > LABEL_LIMIT)
+		{
+			static gchar	short_name[LABEL_LIMIT];
+			gint		h = sizeof short_name / 2, i;
+
+			/* Copy tail of original string. Includes terminator. */
+			for(i = 0; i < h; i++)
+				short_name[sizeof short_name - 1 - i] = text[tl - i];
+			/* Now copy the beginning. */
+			for(i = 0; i < h - 3; i++)
+				short_name[i] = text[i];
+			/* Insert periods to indicate shortened version. */
+			short_name[i++] = '.';
+			short_name[i++] = '.';
+			short_name[i++] = '.';
+			text = short_name;
+		}
 		win_window_set_title(wid, text);
 		gtk_label_set_text(GTK_LABEL(txi->blabel), text);
 	}
